@@ -12,14 +12,17 @@ function addPairHighlighting(elem, depElem) {
     }, false);
 }
 
-function highlightIds(primaryId, secondaryId) {
-    let primaryElem = document.getElementById(primaryId);
-    let secondaryElem = document.getElementById(secondaryId);
+function highlightClasses(primaryClass, secondaryClass) {
+    let primaryElems = document.getElementsByClassName(primaryClass);
+    let secondaryElems = document.getElementsByClassName(secondaryClass);
 
-    if (primaryElem === null || secondaryElem === null) {
+    if (primaryElems.length === 0 || secondaryElems.length === 0) {
         return;
     }
 
+    let primaryElem = primaryElems[0];
+    let secondaryElem = secondaryElems[0];
+
     addPairHighlighting(primaryElem, secondaryElem);
     addPairHighlighting(secondaryElem, primaryElem);
 }
