@@ -1,30 +1,11 @@
 use crate::elf::defs::*;
 use crate::elf::parser::{Note, ParsedElf, ParsedPhdr, ParsedShdr, RangeType};
+use crate::utils;
 use std::fmt::Write;
-use std::path::Path;
 
 const INDENT: &str = "  ";
 const DEFAULT_COLUMNS: usize = 16;
 
-fn basename(path: &str) -> &str {
-    // Wish expect() could use String. This is messy.
-    match Path::new(path).file_name() {
-        Some(name) => name.to_str().unwrap(),
-        None => panic!("basename: failed for path \"{}\"", path),
-    }
-}
-
-fn stem(path: &str) -> &str {
-    match Path::new(path).file_stem() {
-        Some(stem) => stem.to_str().unwrap(),
-        None => panic!("stem: failed for path \"{}\"", path),
-    }
-}
-
-pub fn construct_filename(filename: &str) -> String {
-    stem(basename(filename)).to_string() + ".html"
-}
-
 fn indent(level: usize, line: &str) -> String {
     if line.is_empty() {
         String::new()
@@ -62,7 +43,7 @@ macro_rules! size_dualfmt {
                 "<span class='number' title='{:#x}'>{} ({})</span>",
                 $id,
                 $id,
-                crate::utils::human_format_bytes($id as u64)
+                utils::human_format_bytes($id as u64)
             )
         } else {
             format!("<span class='number' title='{:#x}'>{}</span>", $id, $id)
@@ -93,18 +74,20 @@ macro_rules! wnonl {
     }
 }
 
-fn generate_head(o: &mut String, elf: &ParsedElf) {
+fn generate_head(o: &mut String, elf: &ParsedElf) -> Option<()> {
     let stylesheet: String = include_str!("style.css").indent_lines(3);
     let viewport = "width=900, initial-scale=1";
 
     w!(o, 1, "<head>");
     w!(o, 2, "<meta charset='utf-8'>");
     w!(o, 2, "<meta name='viewport' content='{}'>", viewport);
-    w!(o, 2, "<title>{}</title>", basename(&elf.filename));
+    w!(o, 2, "<title>{}</title>", utils::basename(&elf.filename)?);
     w!(o, 2, "<style>");
     wnonl!(o, 0, "{}", stylesheet);
     w!(o, 2, "</style>");
     w!(o, 1, "</head>");
+
+    Some(())
 }
 
 fn generate_svg_element(o: &mut String) {
@@ -132,8 +115,8 @@ fn generate_svg_element(o: &mut String) {
 fn generate_file_info_table(o: &mut String, elf: &ParsedElf) {
     w!(o, 2, "<table>");
 
-    for (id, desc, value) in elf.information.iter() {
-        wnonl!(o, 3, "<tr id='info_{}'> ", id);
+    for (cl, desc, value) in elf.information.iter() {
+        wnonl!(o, 3, "<tr class='fileinfo_{}'> ", cl);
         wnonl!(o, 0, "<td>{}:</td> ", desc);
         wnonl!(o, 0, "<td>{}</td> ", value);
         w!(o, 0, "</tr>");
@@ -214,16 +197,17 @@ fn generate_phdr_info_tables(o: &mut String, elf: &ParsedElf) {
 fn generate_shdr_info_tables(o: &mut String, elf: &ParsedElf) {
     for (idx, shdr) in elf.shdrs.iter().enumerate() {
         let items = [
-            ("Name", elf.shnstrtab.get(shdr.name)),
-            ("Type", &shtype_to_string(shdr.shtype)),
-            ("Flags", &shflags_to_string(shdr.flags)),
-            ("Vaddr in memory", &hex_dualfmt!(shdr.addr)),
-            ("Offset in file", &hex_dualfmt!(shdr.file_offset)),
-            ("Size in file", &size_dualfmt!(shdr.size)),
-            ("Linked section", &format!("{}", shdr.link)),
-            ("Extra info", &dec_dualfmt!(shdr.link)),
-            ("Alignment", &hex_dualfmt!(shdr.addralign)),
-            ("Size of entries", &size_dualfmt!(shdr.entsize)),
+            ("Index", format!("{}", idx)),
+            ("Name", elf.shnstrtab.get(shdr.name).to_owned()),
+            ("Type", shtype_to_string(shdr.shtype)),
+            ("Flags", shflags_to_string(shdr.flags)),
+            ("Vaddr in memory", hex_dualfmt!(shdr.addr)),
+            ("Offset in file", hex_dualfmt!(shdr.file_offset)),
+            ("Size in file", size_dualfmt!(shdr.size)),
+            ("Linked section", format!("{}", shdr.link)),
+            ("Extra info", dec_dualfmt!(shdr.link)),
+            ("Alignment", hex_dualfmt!(shdr.addralign)),
+            ("Size of entries", size_dualfmt!(shdr.entsize)),
         ];
 
         w!(o, 5, "<table class='conceal itable' id='info_shdr{}'>", idx);
@@ -411,7 +395,7 @@ fn generate_sticky_info_tables(o: &mut String, elf: &ParsedElf) {
 }
 
 fn add_highlight_script(o: &mut String) {
-    let ids = [
+    let classes = [
         "class",
         "data",
         "abi",
@@ -434,8 +418,8 @@ fn add_highlight_script(o: &mut String) {
 
     wnonl!(o, 0, "{}", include_str!("js/highlight.js").indent_lines(3));
 
-    for id in ids.iter() {
-        w!(o, 3, "highlightIds('{}', 'info_{}')", id, id);
+    for cl in classes.iter() {
+        w!(o, 3, "highlightClasses('fileinfo_{}', '{}')", cl, cl);
     }
 
     w!(o, 2, "</script>");
@@ -472,15 +456,21 @@ fn add_arrows_script(o: &mut String, elf: &ParsedElf) {
 
     wnonl!(o, 0, "{}", include_str!("js/arrows.js").indent_lines(3));
 
-    w!(o, 3, "connect('#e_phoff', '#bin_phdr0');");
-    w!(o, 3, "connect('#e_shoff', '#bin_shdr0');");
+    w!(o, 3, "connect('.e_phoff', '.bin_phdr0');");
+    w!(o, 3, "connect('.e_shoff', '.bin_shdr0');");
 
     for i in 0..elf.phdrs.len() {
-        w!(o, 3, "connect('#bin_phdr{} > .p_offset', '#bin_segment{}');", i, i);
+        w!(o, 3, "connect('.bin_phdr{} > .p_offset', '.bin_segment{}');", i, i);
     }
 
     for i in 0..elf.shdrs.len() {
-        w!(o, 3, "connect('#bin_shdr{} > .sh_offset', '#bin_section{}');", i, i);
+        w!(o, 3, "connect('.bin_shdr{} > .sh_offset', '.bin_section{}');", i, i);
+
+        let link = elf.shdrs[i].link;
+
+        if link != 0 {
+            w!(o, 3, "connect('.bin_shdr{} > .sh_link', '.bin_shdr{}');", i, link);
+        }
     }
 
     w!(o, 3, "pushArrowElems();");
@@ -538,35 +528,31 @@ fn append_hex_byte(s: &mut String, byte: u8) {
     }
 }
 
-fn generate_dump_for_byte(idx: usize, dump: &mut String, elf: &ParsedElf) {
-    let byte = elf.contents[idx];
-
-    for range_type in &elf.ranges.data[idx] {
-        if *range_type != RangeType::End {
-            dump.push_str(format!("<span {}>", range_type.span_attributes()).as_str());
-        }
-    }
-
-    if idx < 4 {
-        dump.push_str(&format_magic(byte))
-    } else {
-        append_hex_byte(dump, byte);
-    }
-
-    for _ in 0..elf.ranges.lookup_range_ends(idx) {
-        dump.push_str("</span>");
-    }
-
-    if idx != elf.contents.len() - 1 {
-        dump.push_str(if (idx + 1) % DEFAULT_COLUMNS == 0 { "\n" } else { " " });
-    }
-}
-
 fn generate_file_dump(elf: &ParsedElf) -> String {
     let mut dump = String::new();
 
     for idx in 0..elf.contents.len() {
-        generate_dump_for_byte(idx, &mut dump, elf);
+        let byte = elf.contents[idx];
+
+        for range_type in &elf.ranges.data[idx] {
+            if *range_type != RangeType::End {
+                dump.push_str(format!("<span {}>", range_type.span_attributes()).as_str());
+            }
+        }
+
+        if idx < 4 {
+            dump.push_str(&format_magic(byte))
+        } else {
+            append_hex_byte(&mut dump, byte);
+        }
+
+        for _ in 0..elf.ranges.lookup_range_ends(idx) {
+            dump.push_str("</span>");
+        }
+
+        if idx != elf.contents.len() - 1 {
+            dump.push_str(if (idx + 1) % DEFAULT_COLUMNS == 0 { "\n" } else { " " });
+        }
     }
 
     dump
@@ -577,7 +563,7 @@ fn generate_ascii_dump(o: &mut String, elf: &ParsedElf) {
         if b.is_ascii_graphic() {
             let ch = *b as char;
 
-            match crate::utils::html_escape(ch) {
+            match utils::html_escape(ch) {
                 Some(escaped) => {
                     wnonl!(o, 0, "{}", escaped);
                 }
@@ -630,7 +616,7 @@ fn generate_body(o: &mut String, elf: &ParsedElf) {
     w!(o, 1, "</body>");
 }
 
-pub fn generate_report(elf: &ParsedElf) -> String {
+pub fn generate_report(elf: &ParsedElf) -> Option<String> {
     let mut output = String::new();
 
     w!(&mut output, 0, "<!doctype html>");
@@ -641,5 +627,5 @@ pub fn generate_report(elf: &ParsedElf) -> String {
 
     w!(&mut output, 0, "</html>");
 
-    output
+    Some(output)
 }
